from ConfigParser import SafeConfigParser
import os

class ConfigParser(SafeConfigParser):
    optionxform = str

class Configuration(object):
    def __init__(self, engine):
        self.engine = engine

    # a mixin to deal with persistent properties
    def record(self):
        # write out a k/v pair to some stable storage
        # we need to open a config parser for the
        # project and the global
        world = ConfigParser()
        project = ConfigParser()

        # We want to be sure that we only record values
        # for things that have registered questions
        # not just properties in the namespace
        questions = set()
        for p in self.engine.plugins.itervalues():
            for q in p.questions:
                if not q.isCallback and q.scope is not None:
                    questions.update([q.action])

        valid = lambda x: x in questions

        if not world.has_section("default"):
            world.add_section("default")
        for k, v in self.engine.properties.items():
            if valid(k):
                world.set("default", str(k), str(v))

        for plugin in self.engine.plugins.values():
            if not project.has_section(plugin.__name__):
                project.add_section(plugin.__name__)
            for k, v in plugin.properties.items():
                if valid(k):
                    project.set(plugin.__name__, k, str(v))

        if not project.has_section("default"):
            project.add_section("default")
        for k, v in self.engine.project.properties.items():
            if valid(k):
                project.set("default", k, str(v))

        wfp = open(os.path.expanduser('~/.skeletor'), 'w')
        pfp = open(os.path.join(self.engine['base'],
                                ".skeletor"), 'w+')
        world.write(wfp)
        project.write(pfp)
        wfp.close()
        pfp.close()



    def load(self):
        # this should load all the state
        # from a config file at a given scope
        # the goal is to populate a properties dict
        # we need to read these in two passes to make sure globals get
        # assigned
        world = ConfigParser()
        project = ConfigParser()

        world.read(os.path.expanduser('~/.skeletor'))
        project.read(os.path.join(self.engine['base'], ".skeletor"))

        for section in world.sections():
            for k,v in world.items(section):
                self.engine[k] = v

        plugins = self.engine.plugins
        sections = list(project.sections())
        try:
            sections.remove("default")
            pe = self.engine.project
            for k,v in project.items("default"):
                pe[k] = v
        except ValueError:
            pass

        for section in sections:
            # look up if the section relates to the project
            # or plugin by name
            plugin = plugins.get(section, None)
            if plugin is None:
                plugin = self.engine.project

            for k,v in project.items(section):
                plugin[k] = v




