from registry import PROJECT, PLUGIN, WORLD, TEMPORARY, CONFIG_ROOT

_marker = object()

# The model is that we offer static scopes and static name resolution
# through those scopes
#  [World -> *[Project -> *[Plugins]]]
# There are global settings (world) and each project will have its own
# settings as well as storing project specific data.
# Remember that lower scopes can explicitly store data in the
# higher scopes, so if plugins maintain global data they stuff it in
# world.

# A plugin looking up a variable will search its own space, then its
# projects, then world, unless a scope is passed into the resolve
# mechanism

# The root object will contain the registry
# which can resolve all the names w/o issue

class Entity(object):
    """A conceptual object in the project. There will be more than one
    view on most of these. For example a 'content type' entity will
    produce an implementation for the content type, tests for it maybe
    some docs.

    Plugins manage the mapping between Entities and their File
    renditions.
    """
    def __init__(self, parent=None):
        self.parent = parent

    def _getRoot(self):
        if self.parent: return self.parent.root
        return self
    root = property(_getRoot)

    def _getName(self):
        return self.__name__

    def _setName(self, value):
        self.__name__ = value

    name = property(_getName, _setName)

    def _getKeyspace(self):
        return "/%s/%s" % ( CONFIG_ROOT,
                            TEMPORARY
                            )
    keyspace = property(_getKeyspace)

    def resolve(self, key, scope=None):
        return self.root.registry.resolve(key, scope)

    def get(self, key, default=None, scope=None):
        try:
            v = self.resolve(key, scope)
            if not v: v = default
        except:
            v = default
        return v

    def set(self, key, value, **kw):
        # map the scope
        self.root.registry.set(key, value, scope=self.keyspace, **kw)


class Project(Entity):

    def __init__(self, engine):
        Entity.__init__(self, engine)
        self.engine = engine
        self._model = {}
        self.scope = PROJECT

    def render(self, plugin, template):
        # The template will contain the metadata needed to determine
        # its mapping to the output
        fp = open(template.properties['target'], "wcU")
        template.render(plugin, fp)
        fp.close()
        self.engine.report.info("Rendered %s" % fp.name)

    def _getKeyspace(self):
        return "/%s/projects/project[@name='%s']" % ( CONFIG_ROOT,
                                                      self.name
                                                      )
    keyspace = property(_getKeyspace)
