from skeletor import widgets
from skeletor.utils import walkPluginDir, normalizePath, loadTemplate
from skeletor import template
from skeletor.plugin import Command
from skeletor.question import GLOBAL, PROJECT, PLUGIN
from skeletor.plugins.base import Base

import os

testTemplate = template.PythonTemplate(
    """
    def test_#{ContentClass,code,replace}#(self):
        c = makeContent(self.folder, 'content', '#{ContentMetaType,code,replace}#')
        assert c

        # assert that content can render itself
        assert c.view()

""")

class Content(Base):
    """Add a content type to a project

    This will add a new content type, register it for initilization
    and add a supporing test skeleton.
    """
    __name__= "content_type"
    _datadir = None

    @property
    def contentTemplate(self):
        if not hasattr(self, '_template'):
            self._template = loadTemplate(self._datadir)
        return self._template

    def commands(self, commands):
        commands['add_content'] = Command('add_content', self.create,
                 """[content type] Add a content type to the base project""")



    def options(self, parser): pass

    def LoadConfig(self, project):
        Base.LoadConfig(self, project)
        self.addQuestion(widgets.entry,
                         "Content Type Name", "ContentClass",
                         required=True,
                         scope=None,
                         ).ask()

        self.addQuestion(widgets.entry,
                         "Content Meta Type", "ContentMetaType",
                         default = self['ContentClass'],
                         required=True,
                         scope=None,
                         )


    def Transform(self, project):
        # For each entity in base's model add the correct license
        # snippet if there is one.
        # XXX: We will ignore fragment for now
        # add the license file

        # Add the import for the type
        importTemplate = template.PythonTemplate("\nimport %s" % \
                                                 self['ContentClass'])
        self.fillSlotAt("content/__init__.py",
                        "ModuleLevelCode",
                        importTemplate)

        # Add a test to the project
        self.fillSlotAt("tests/test_project.py",
                        'ProjectTest',
                        testTemplate)

        # and add the content itself
        self.addTemplateAt(self.contentTemplate, "content/%s.py" % \
                           self['ContentClass'])

        for ct in self.templates.values():
            self.engine.project.render(self, ct)


    def create(self, project, *args):
        if len(args):
            self['ContentClass'] = args[0]
            args = args[1:]
        self(self.engine.project, *args)

