from skeletor import widgets
from skeletor.utils import walkPluginDir, normalizePath, loadTemplate
from skeletor import template
from skeletor.plugin import Command
from skeletor.question import GLOBAL, PROJECT, PLUGIN
from skeletor.plugins.base import Base
import os

class LicensePack(object):
    def __init__(self, name, license, datadir, copyright=None, fragment=None):
        # names of files that contain the templates for a given lic.
        self._datadir = datadir
        self.__name__ = name
        self.license = loadTemplate(license, self._datadir)
        if copyright is not None: copyright = loadTemplate(copyright, self._datadir)
        self.copyright = copyright
        if fragment: fragment = loadTemplate(fragment, self._datadir)
        self.fragment = fragment

    def __repr__(self):
        return "<LicensePack %s %s>" %(self.__name__, self.license)


class License(Base):
    """execute a license pass over the model"""
    __name__= "license"

    @property
    def licenses(self):
        if not hasattr(self, '_licenses'):
            self._licenses = {
                'GPL' : LicensePack("GPL", "LICENSE.GPL", self._datadir, None, None),
                'ZPL' : LicensePack("ZPL", "LICENSE.ZPL", self._datadir,
                                    "COPYRIGHT.ZPL", "fragment.ZPL"),
                }
        return self._licenses

    def __init__(self, engine):
        Base.__init__(self, engine)

    def commands(self, commands):
        commands['license'] = Command('license', self.license,
                """[licence]  Spread gooey license love all over your project""")

    def options(self, parser):
        pass

    def LoadConfig(self, project):
        Base.LoadConfig(self, project)
        self.addQuestion(widgets.checkbox,
                         "License", "ProductLicense",
                         options=licenses.keys(),
                         column="License Choice",
                         single=True,
                         required=True,
                         ).ask()

        self.license = licenses[self['ProductLicense']]


    def Transform(self, project):
        # For each entity in base's model add the correct license
        # snippet if there is one.
        # XXX: We will ignore fragment for now
        # add the license file
        path = self.addTemplateAt(self.license.license, "LICENSE")
        if os.path.exists(path): os.remove(path)

        if self.license.copyright:
            path = self.addTemplateAt(self.license.copyright, "COPYRIGHT.txt")
            if os.path.exists(path): os.remove(path)

        for ct in self.templates.values():
            self.engine.project.render(self, ct)


    def license(self, project, *args):
        if len(args):
            self['ProductLicense'] = args[0]
            args = args[1:]
        self(self.engine.project, *args)

