
# scopes
from registry import PLUGIN, PROJECT, WORLD, TEMPORARY
GLOBAL = WORLD

class Question(object):
    def __init__(self, plugin, kind, question, action,
                 scope=TEMPORARY,
                 flag=None,
                 actionname=None,
                 **kwargs):
        """Question
            plugin -- plugin owning this question
            kind -- a widget
            question -- question test
            action -- if string - name of variable to assign to in the
                      plugins question
                      if callable - callback will be called with the result
        """
        self.plugin = plugin
        self.kind = kind
        self.question = question
        self.options = kwargs
        self.action = action
        self.isCallback = not isinstance(action, basestring)
        self.scope = scope
        self.flag = flag
        if not actionname:
            self.actionname = action
        else:
            self.actionname = actionname

    def check(self):
        result = None
        if self.flag: # this maps to a cmdline flag
            result = getattr(self.plugin.engine.options, self.flag, None)

        registry = self.plugin.engine.registry
        if self.isCallback: action = self.actionname
        else: action = self.action
        if not result:
            # is the stored value available
            result = registry.get(action)
        if result is not None:
            self.record(action, result)

        return result

    def ask(self):
        # Figure out if we already know the answer to the question
        # or ask it
        # we do this via
        # cmdline flag
        # stored value
        # asking..
        result = self.check()
        if result: return result

        # Ask the question
        result = self.kind(self.plugin, self.question, **self.options)
        if self.isCallback: self.action(result, self)
        else: self.record(self.action, result)
        return result

    def record(self, key, value):
        """At this questions scope we are going to add an entry to the
        recorded data. This will let us skip asking some questions
        between projects
        """
        registry = self.plugin.engine.registry
        #switch on the scope and record the property
        registry.set(key, value, scope=self.scope)
##         if not self.scope  or self.scope == TEMPORARY:
##             registry.set(key, value)
##         elif  self.scope == PROJECT:
##             self.plugin.engine.project.set(key, value)
##         elif self.scope == PLUGIN:
##             self.plugin.set(key, value)
##         else:
##             self.plugin.engine.set(key, value)


