import unittest
from plugin import Plugin, Command
from engine import Engine
from report import Report

class TestReport(unittest.TestCase):
    """Test the reporting module"""
    def test_logMsg(self):
        # we just expose the standard logging module
        # configured to handle the skeletor log
        report = Report()
        assert report.info
        assert report.error

class Base(Plugin):
        """Base model handler"""
        __name__= "base"

        def options(self, parser):
                parser.add_option('-q', '--quit',
                                  action="callback",
                                  callback=self.quit,
                                  help="quit this puppy")

        def quit(self, option, opt, value, parser):
                pass


        def commands(self, commands):
                commands['clone'] = Command('clone', self.clone,
                """clone  -- Clone a skeleton tree""")

        def clone(self, *args):
                pass

class TestPlugin(unittest.TestCase):
    def test_plugins(self):
        # This test doesn't use the module autoloader, but
        # does verify that a manually loaded plugin can
        # populate the proper structures
        e = Engine()
        e.addPlugin(Base, 'base')
        assert e.plugins
        assert 'clone' in e.commands


if __name__ == "__main__":
    suite = unittest.makeSuite(TestReport)
    suite.addTests(unittest.makeSuite(TestPlugin))

    import test_template
    import test_setup
    suite.addTests(test_template.suite())
    suite.addTests(test_setup.suite())
    unittest.TextTestRunner(verbosity=1).run(suite)

