"""
>>> from registry import *
>>> r = Registry()
>>> r.load("tests/a.xml", "tests/b.xml")

>>> assert r.resolve("/skeletor/plugins/plugin[@name='zub']/author") == 'bcsaller'
>>> assert r.resolve("/skeletor/plugins/plugin[@name='base']/key") == 'value'
>>> assert r.resolve("/skeletor/plugins/plugin[@name='base']/sourcedir") == "file:///tmp/foo"



SCOPED RESOLUTION
we should be able to resolve a k/v pair from world
>>> assert r.resolve("copyright") == "GPL"

But we can't resolve 'key' because we haven't established _which_
project gets searched for the project scope
>>> r.resolve("key")
Traceback (most recent call last):
...
KeyError: ... key'

Testing the same key with a properly set scope would yield a value,
this is shown later in this file.

We can set values as expected, though in typical usage a scope would
also be passed in to explicitly set how to persist a value. In these
cases the values are temporary and will not be persisted, but can
resolve normally in the meantime.
>>> r.set("another", "item")
>>> assert r.resolve("another") == "item"
>>> r.set("one", "more")
>>> assert r.resolve("one") == "more"

Create a new registry and test that we can get/set properly scoped
variables.

>>> r = Registry()
>>> r.setScope(PROJECT, "/skeletor/projects/project[@name='test']")
>>> r.setScope(PLUGIN, "/skeletor/plugins/plugin[@name='alpha']")
>>> r.set("testing", "new value", PROJECT)
>>> r.set("foo", "bar", PLUGIN)

This should be a TEMPORARY value
>>> r.set("a", "b")

And this is global/world

>>> r.set("skeletor", "works", WORLD)

And we can resolve element from each scope as needed, because of the
setScope calls automatic resolution happens as one might expect.
>>> assert r.get("skeletor") == "works"
>>> assert r.get("foo") == "bar"
>>> assert r.get("testing") == "new value"
>>> assert r.get("a") == "b"
"""
import unittest
import os


__author__ = 'Benjamin Saller <bcsaller@objectrealms.net>'
__docformat__ = 'restructuredtext'
__copyright__ = 'Copyright ObjectRealms, LLC, 2005'
__license__  = 'The GNU Public License V2+'



if __name__ == "__main__":
    import doctest, pdb
    options = doctest.ELLIPSIS
    #options |= doctest.REPORT_ONLY_FIRST_FAILURE
    doctest.testmod(optionflags=options)

