import os
import zipfile
import inspect
from config import RESOURCE_DIR, RESOURCE_ZIP, CONFIG_DIR
import pkg_resources as pkgr
from setuptools.archive_util import unpack_zipfile 

_marker = object()

def getModule(modpath):
    """Get a module in the 'package.module' format"""
    m = __import__(modpath, globals(), locals(), [modpath])
    assert inspect.ismodule(m)
    return m

def loadTemplate(base, datadir):
    if not base: return
    import pdb; pdb.set_trace()
    fname = os.path.join(datadir, base)
    d = open(fname, "r").read()
    t = template.FileTemplate(d, file_name=fname)
    return t

def getClass(classpath, data=None):
    """Get a class object from a path in the "package.module.Class"
    format.
    """
    #XXX: python2.4
    #classpath, className = classpath.rsplit('.', 1)
    p = classpath.split('.')
    classpath, className = ".".join(p[:-1]), p[-1]
    module = __import__(classpath, globals(), locals(), className)
    klass = getattr(module, className)
    assert inspect.isclass(klass)
    if data:
        klass._datadir = data
    return klass

def offsetpath(path, offset):
    parts = path.split(os.path.sep)[offset:]
    return os.path.sep.join(parts)


removePluginDir = offsetpath


def unlink(name):
    if os.path.exists(name):
        os.unlink(name)
    return name


def normalizePath(path, destbase, offset):
    p = removePluginDir(path, offset)
    d = os.path.join(destbase, p)
    return d


ignore_svn = lambda name: name.find('.svn')>-1


def make_zip(target, dest, zipname, truncate=True, ignore=ignore_svn):
    pdir=os.path.dirname(target)
    zippy = zipfile.ZipFile(os.path.join(dest, zipname), 'w')
    for name, dirs, files in os.walk(target):
        if ignore(name):
            continue
        for dfile in files:
            dfile=os.path.join(name, dfile)
            arcname=dfile
            if truncate:
                offset = pdir.count(os.path.sep) +1
                arcname=offsetpath(dfile, offset)
            zippy.write(dfile, arcname)
    zippy.close()

def copy(source, dest):
    os.system("cp %s %s" %(source, dest))

def walkPluginDir(src, dest):
    """Yield up each filename with its type and target location"""

    default = src.__module__
    src = getattr(src, '_datadir',  default)
    if not src: src = default

    # if a zip resource exist, choose it
    # create a namespace directory for it
    # in user space
    if pkgr.resource_exists(src, RESOURCE_ZIP):
        src = pkgr.resource_filename(src, RESOURCE_ZIP)
        nsdir = os.path.join(CONFIG_DIR, default)
        if not os.path.exists(nsdir):
            os.mkdir(nsdir)
        unpack_zipfile(src, nsdir)
        # someway to identify whether this was
        # this was dev mode would be good / set reg
        # to local dire
        src = os.path.join(nsdir, RESOURCE_DIR)
    elif pkgr.resource_exists(src, RESOURCE_DIR):
        src = pkgr.resource_filename(src, RESOURCE_DIR)

    base = os.path.abspath(dest)
    offset = src.count(os.path.sep) +1
    for name, dirs, files in os.walk(src):
        for d in dirs:
            full = os.path.join(name, d)
            yield ("directory", full, normalizePath(full, base, offset))

        for f in files:
            full = os.path.join(name, f)
            yield ("file", full, normalizePath(full, base, offset))
            
    
def cloneTree(dest, src, includeFiles=False):
    """Copy all the directories from one location to the dest dest"""
    for kind, full, target in walkPluginDir(src, dest):
        if kind == "directory":
            if not os.path.exists(target):
                try: os.makedirs(target)
                except OSError: pass
        #elif includeFiles:
        #        copy(full, target)


